-module(event).

-record(state, {server, name = "", to_go = 0}).

-export([init/3, start/2, start_link/2]).

start(EventName, Delay) ->
    spawn(?MODULE, init, [self(), EventName, Delay]).

start_link(EventName, Delay) ->
    spawn_link(?MODULE, init, [self(), EventName, Delay]).

init(Server, EventName, Delay) ->
    loop(#state{server = Server, name = EventName,
		to_go = normalize(Delay)}).

normalize(N) ->
    Limit = 49 * 24 * 60 * 60,
    [N rem Limit | lists:duplicate(N div Limit, Limit)].

loop(S = #state{server = Server, to_go = [T | Next]}) ->
    receive
      {Server, Ref, cancel} -> Server ! {Ref, ok}
    after T * 1000 ->
      case Next of
        [] -> Server ! {done, S#state.name};
        _ -> loop(S#state{to_go = Next})
      end
    end.
