package day3
import akka.actor.Actor;
import akka.pattern.ask;
import akka.actor.ActorSystem;
import akka.actor.Props
import akka.util.Timeout;
import scala.io.Source
import scala.concurrent.duration._;
import scala.concurrent.{Await, ExecutionContext, Future}
import scala.xml.XML;


case class Get(url : String)

object Fetch {
  def getPageSize(url: String) = Source.fromURL(url).mkString.length()

  def getLinks(url: String) = {
    val content = Source.fromURL(url).mkString;
    "<a".r.findAllMatchIn(content).length
  }
}

class PageLoader extends Actor {

  def receive: Actor.Receive = {
    case Get(url) => sender ! Fetch.getLinks(url)
  }
}

object LoadPages extends App {
  val system = ActorSystem("ActOne")
  val urls = List(
    "https://www.orbit.fm",
    "https://jxs.me",
    "https://www.example.com",
    "https://sdf.org"
  )


  def timeMethod(method: () => Unit) = {
    val start = System.nanoTime
    method()
    val end = System.nanoTime
    println("Method took " + (end - start) / 1000000000.0 + " seconds.")
  }

  def getPageSizeSequentially() = {
    for (url <- urls) {
      println("Links for " + url + ": " + Fetch.getLinks(url))
    }
  }

  def getPageSizeConcurrently() = {
    implicit val timeout = Timeout(5.seconds)
    val futures = urls.map(url => {
      val act = system.actorOf(Props[PageLoader], name = url.replace("/", ""))
      act ? Get(url)
    })
    for (future <- futures) {
      val result = Await.result(future, timeout.duration)
      println(result)
    }
  }

  timeMethod { getPageSizeSequentially }
  timeMethod { getPageSizeConcurrently }
}
